const CommandExecutor = require("../CommandExecutor");
const IPMISetup = require("../ServerRegistering/IPMISetup");
const ServerInfoGatherer = require("../ServerInfoGatherer").ServerInfoGathererSingleton;
const EFIbootmgr = require("../ServerInfoGatherer/Commands").EFIbootmgr;
const AgentIdentity = require("../../Agent/Identity");

class BDKServerCleanup
{
	constructor()
	{
	}


	/**
	 * @param {agentIdentity} agentIdentity
	 * @param {Object} objServer
	 * @param {Array<Object>} arrServerInterfaces
	 * @param {Array} objIPOOB
	 * @param {Array} arrDNSServers
	 * @param {Array} arrNTPServers
	 * @returns {string}
	 */
	static async run(agentIdentity, objServer, arrServerInterfaces, objIPOOB, arrDNSServers, arrNTPServers)
	{
		let strCommandOutput = "";

		// if(
		// 	objServer["server_vendor"] === BDKServerCleanup.HP_VENDOR 
		// 	|| objServer["server_vendor"] === BDKServerCleanup.HPE_VENDOR
		// )
		// {
		// 	strCommandOutput += await BDKServerCleanup._runServerCleanupAuxiliaryFunctionsBatch();
		// }

		strCommandOutput += await BDKServerCleanup._runCleanupBatch(agentIdentity, objServer, arrServerInterfaces, objIPOOB, arrDNSServers, arrNTPServers, objServer.NICDetails);
		
		return strCommandOutput;
	}
	

	/**
	 * @param {agentIdentity} agentIdentity
	 * @param {Object} objServer
	 * @param {Array<Object>} arrServerInterfaces
	 * @param {Array} objIPOOB
	 * @param {Array} arrDNSServers
	 * @param {Array} arrNTPServers
	 * @param {Array} arrNICDetails
	 * @returns {string}
	 */
	static async _runCleanupBatch(agentIdentity, objServer, arrServerInterfaces, objIPOOB, arrDNSServers, arrNTPServers, arrNICDetails)
	{
		// Make sure that objServer has the neccesary information
		objServer = {
			...(await ServerInfoGatherer.gatherAllInfo()),
			...objServer
		};

		let strCommandOutput = "";

		if(objServer["server_disk_wipe"])
		{
			strCommandOutput += await BDKServerCleanup.diskCleanup(
				agentIdentity,
				objServer,
				false
			);
		}

		strCommandOutput += await BDKServerCleanup.networkingIntelCleanup(objServer);
		strCommandOutput += await BDKServerCleanup.serverInterfacesCleanup(arrServerInterfaces);
		strCommandOutput += await BDKServerCleanup.IPMIUsersAndNetworkingCleanup(agentIdentity, objServer, objIPOOB, arrDNSServers, arrNTPServers);
		strCommandOutput += await BDKServerCleanup.efibootmgrCleanup(arrNICDetails);

		return strCommandOutput;
	}


	/**
	 * @param {Array<Object>} arrServerInterfaces
	 * @returns {string}
	 */
	static async serverInterfacesCleanup(arrServerInterfaces)
	{
		let strInterfacesCleanupCommand = "";
		
		for(let objServerInterface of arrServerInterfaces)
		{
			if(objServerInterface["server_interface_add_on_type"] === BDKServerCleanup.SERVER_INTERFACE_ADD_ON_HBA)
			{
				const strLinedMAC = objServerInterface["server_interface_add_on_mac"].replace(/:/g, "-");
				strInterfacesCleanupCommand += `hbacmd showtarget ${strLinedMAC} | grep "Target iSCSI Name:" | awk '{print "hbacmd RemoveTarget ${strLinedMAC} \\"" $4 "\\""}' | sh\n`;
			}
		}

		return await CommandExecutor.execCommandProcess(strInterfacesCleanupCommand);
	}


	/**
	 * @param {agentIdentity} agentIdentity
	 * @param {Object} objServer
	 * @param {Object} objIPOOB
	 * @param {Array} arrDNSServers
	 * @param {Array} arrNTPServers
	 * @param {Encryption} serverEncryption
	 * @returns {string}
	 */
	static async IPMIUsersAndNetworkingCleanup(agentIdentity, objServer, objIPOOB, arrDNSServers, arrNTPServers, serverEncryption)
	{
		let strIPMIAndNetworkingCleanupCommand = `
			ILOREST="/sbin/ilorest"
			IPMITOOL="/usr/bin/ipmitool"
			HPRCU="/sbin/hprcu"
		`;
		

		if(arrNTPServers && arrNTPServers.length !== 0)
		{
			const strServer = arrNTPServers[0];
			strIPMIAndNetworkingCleanupCommand += `
				ntpdate ${strServer}
				hwclock --adjust
				hwclock --systohc
			`;
		}
		else
		{
			throw new Error("Please provide at least one NTP server.");
		}

		if (!arrDNSServers || arrDNSServers.length === 0)
		{
			throw new Error("Please provide at least one DNS server.");
		}

		strIPMIAndNetworkingCleanupCommand += `
			IPMITOOL=/usr/bin/ipmitool
			ipmichannel=${objServer["server_ipmi_channel"]}
		`;

		if(
			objServer["server_vendor"] === BDKServerCleanup.HP_VENDOR 
			|| objServer["server_vendor"] === BDKServerCleanup.HPE_VENDOR
		)
		{
			let bIsCartridge = false;

			if(
				typeof objServer["server_product_name"] === "string" 
				&& objServer["server_product_name"].includes("Server Cartridge")
			)
			{
				bIsCartridge = true;
			}

			if(!bIsCartridge)
			{
				let [strFirstDNSServer, strSecondDNSServer, ...restDNS] = arrDNSServers;
				if (!strSecondDNSServer)
				{
					strSecondDNSServer = strFirstDNSServer;
				}

				let [strFirstNTPServer, strSecondNTPServer, ...restNTP] = arrNTPServers;
				if (!strSecondNTPServer)
				{
					strSecondNTPServer = strFirstNTPServer;
				}

				let strTimezoneID = "";

				if (objServer["server_vendor"] === BDKServerCleanup.HP_VENDOR)
				{
					strIPMIAndNetworkingCleanupCommand += "ILOREST_JSON_LOCATION=/opt/hp/ilorest_jsons/others\n";
					strTimezoneID = "434";
				}
				
				if (objServer["server_vendor"] === BDKServerCleanup.HPE_VENDOR)
				{
					strIPMIAndNetworkingCleanupCommand += "ILOREST_JSON_LOCATION=/opt/hp/ilorest_jsons/gen10\n";
					strTimezoneID = "14";
				}

				strIPMIAndNetworkingCleanupCommand += `
					sed -e 's/%HOSTNAME%/${objIPOOB["server_ipmi_host"].replace(/\./g, "-")}/g' -e 's/%DNS1%/${strFirstDNSServer}/g' -e 's/%DNS2%/${strSecondDNSServer}/g' -e 's/%IP_ADDRESS%/${objIPOOB["server_ipmi_host"]}/g' -e 's/%GATEWAY%/${objIPOOB["subnet_oob_gateway_human_readable"]}/g' -e 's/%NETMASK%/${objIPOOB["subnet_oob_netmask_human_readable"]}/g' \${ILOREST_JSON_LOCATION}/ilo_configure_network.json > /tmp/ilo_configure_network.json
					sed -e 's/%NTP1%/${strFirstNTPServer}/g' -e 's/%NTP2%/${strSecondNTPServer}/g' -e 's/%TZ%/${strTimezoneID}/g' \${ILOREST_JSON_LOCATION}/ilo_timezone_and_ntp_servers.json >/tmp/ilo_timezone_and_ntp_servers.json
					sed -e 's/%KEY%/3JW5ZVM4RTVDCNP53CZQ8PHW2/g' \${ILOREST_JSON_LOCATION}/ilo_add_license.json > /tmp/ilo_add_license.json;
					sed -e 's/%LOGIN%/${objServer["server_ipmi_internal_username"]}/g' -e 's/%PASSWORD%/${objServer["server_ipmi_internal_password_decrypted"].toString("ascii")}/g' \${ILOREST_JSON_LOCATION}/ilo_add_user.json >/tmp/ilo_add_user.json
				`;

				// add the definition of ilo_login
				strIPMIAndNetworkingCleanupCommand += `
					function ilo_login() {
						logincount=1
						while true; do
							timeout --kill-after=13 10 \${ILOREST} login >/dev/null 2>&1
							if [ $? -eq 0 ]; then
								echo 'Login successfull'
								break
							fi
							echo \"Waiting for ILO to be available, try #\${logincount}...\"
							sleep 5
							logincount=\$((logincount+1))
							if [ \${logincount} -eq 40 ]; then
								echo \"Unable to login on iLO. Exiting\"
								exit
							fi
						done
					}
				`;

				strIPMIAndNetworkingCleanupCommand += `
					#reset to factory defaults
					ilo_login
					echo 'Reseting ILO to factory defaults'
					\${ILOREST} rawpost \${ILOREST_JSON_LOCATION}/ilo_reset_to_factory_defaults.json #iloreset automatically
					sleep 5 #just in case ILO did not begin the reset procedure
					
					#disable DHCP and set ip address
					ilo_login
					echo 'Configuring ILO network'
					\${ILOREST} rawpatch /tmp/ilo_configure_network.json #requires iloreset

					#set timezone
					ilo_login
					echo 'Configuring ILO timezone and NTP servers'
					\${ILOREST} rawpatch /tmp/ilo_timezone_and_ntp_servers.json # iloreset required

					#add ILO license
					ilo_login
					echo 'Adding the ILO license'
					\${ILOREST} rawpost /tmp/ilo_add_license.json #iloreset not required
					
					#disable ILO federation
					ilo_login
					echo 'Disable ILO federation'
					\${ILOREST} rawpatch \${ILOREST_JSON_LOCATION}/ilo_disable_federation.json #does not required
					
					#configure authentication settings
					ilo_login
					echo 'Configure ILO authentication settings'
					\${ILOREST} rawpatch \${ILOREST_JSON_LOCATION}/ilo_authentication_settings.json #iloreset
					
					#add BSI administrative user
					ilo_login
					echo 'Add BSI administrative user'
					\${ILOREST} rawpost /tmp/ilo_add_user.json #iloreset not required #iloreset not required
					
					#disable administrator
					ilo_login
					echo 'Disable default ILO administrator account'
					\${ILOREST} rawpatch \${ILOREST_JSON_LOCATION}/ilo_disable_administrator.json #iloreset not required
					
					#cleanup
					ilo_login
					echo 'Clearing AHS logs'
					\${ILOREST} rawpost \${ILOREST_JSON_LOCATION}/ilo_clear_ahs.json #requires iloreset
					
					#configure other ILO settings
					ilo_login
					echo 'ILO general settings'
				`;

				// Temporary, as it needs to be taken to prod as a hotpatch. Needs to be added to datacenter config.
				if (agentIdentity.environment === "production")
				{
					strIPMIAndNetworkingCleanupCommand += "\${ILOREST} rawpatch \${ILOREST_JSON_LOCATION}/ilo_other_settings.json #does not iloreset automatically anymore, needs to be explicit";
				}
				else
				{
					strIPMIAndNetworkingCleanupCommand += "\${ILOREST} rawpatch \${ILOREST_JSON_LOCATION}/ilo_other_settings.json #does not iloreset automatically anymore, needs to be explicit";
				}

				strIPMIAndNetworkingCleanupCommand += `
					ilo_login
					\${ILOREST} iloreset
					sleep 5s
					ilo_login
				`;
			}
			else
			{
				// cartridges don't require any iLO work at this moment, as they don't have their own or users to be added
			}

			return await CommandExecutor.execCommandProcess(strIPMIAndNetworkingCleanupCommand);
		}
		else
		{
			const strIPMIToolLocation = "IPMITOOL=/usr/bin/ipmitool";
			const nIPMIChannel = objServer["server_ipmi_channel"];

			if(objServer["server_status"] === "cleaning")
			{
				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL lan set ${nIPMIChannel} ipsrc static
				`);

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL lan set ${nIPMIChannel} ipaddr ${objIPOOB["server_ipmi_host"]}
				`);

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL lan set ${nIPMIChannel} netmask ${objIPOOB["subnet_oob_netmask_human_readable"]}
				`);

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL lan set ${nIPMIChannel} defgw ipaddr ${objIPOOB["subnet_oob_gateway_human_readable"]}
				`);

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL lan set ${nIPMIChannel} access on
				`);

				try
				{
					await CommandExecutor.execCommandProcess(
						strIPMIToolLocation + `
						$IPMITOOL user disable 4
					`);
				}
				catch(error)
				{
					console.error(error.stack);
				}

				// disabling default administrator user
				try
				{
					await CommandExecutor.execCommandProcess(
						strIPMIToolLocation + `
						$IPMITOOL user disable ${IPMISetup.DEFAULT_ADMINISTRATOR_USER_ID}
					`);
				}
				catch(error)
				{
					console.error(error.stack);
				}

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL user enable ${IPMISetup.INTERNAL_ADMINISTRATOR_USER_ID}
				`);

				await CommandExecutor.execCommandProcess(
					strIPMIToolLocation + `
					$IPMITOOL bmc reset cold
				`);

				// Waiting for IPMI stack initialization non-HP
				await new Promise((resolve, reject) => {
					setTimeout(
						resolve,
						25000 //ms
					);
				});
			}
		}

		return "";
	}


	/**
	 * @param {Array} arrNICDetails
	 * @returns {string}
	 */
	static async efibootmgrCleanup(arrNICDetails)
	{
		let strBootmgrCleanupCommand = "";
		const objNICMatches = {};
		const objBiosInfo = await ServerInfoGatherer.gatherBiosInfo();

		if(
			objBiosInfo.enabled_efi_variables === false 
			|| (
				!(await ServerInfoGatherer.objUnprocessedServerInfo)["EFIbootmgr"]
				|| (await ServerInfoGatherer.objUnprocessedServerInfo)["EFIbootmgr"].enabled_efi_variables === false
			)
		)
		{
			return "";
		}

		const EFIbootmgrInstance = new EFIbootmgr({bMustBeFiltered: false, bMustBeParsed: true});
		await EFIbootmgrInstance.execCommand();

		const strEfibootmgrInformation = EFIbootmgrInstance.output;
		
		if(typeof strEfibootmgrInformation !== "string")
		{
			console.error("Returned efibootmgr information is not a string.");
			return "";
		}

		const arrEfibootmgrRows = strEfibootmgrInformation.split("\n");

		for(let strRow of arrEfibootmgrRows)
		{
			const arrRowElements = strRow.split("\t");

			if(arrRowElements.length !== 2)
			{
				continue;
			}

			const arrMatches = arrRowElements[0].match(/Boot[0-9A-F]+/g);

			if (arrMatches === null)
			{
				continue;
			}

			const strBootOptionNumber = arrMatches[0].replace(/Boot/g, "");

			if(arrRowElements[1].includes("File(") && arrRowElements[1].includes(".efi"))
			{
				strBootmgrCleanupCommand += `efibootmgr -b ${strBootOptionNumber} -B\n`;
			}
			else
			{
				for(let strMAC in arrNICDetails)
				{
					const strStrippedMAC = strMAC.toLowerCase().replace(/:/g, "");

					if(arrRowElements[1].includes(strStrippedMAC))
					{
						if(
							arrRowElements[0].includes("PXE")
							&& arrRowElements[0].includes("IPv4")
						)
						{
							objNICMatches[strMAC] = strBootOptionNumber;
							strBootmgrCleanupCommand += `efibootmgr -b ${strBootOptionNumber} -a\n`;
						}
						else
						{
							// Currently, only IPv4 PXE needs to be active, as we don't have support for anything else, so we deactivate other options for the MAC.
							strBootmgrCleanupCommand += `efibootmgr -b ${strBootOptionNumber} -A\n`;
						}

						break;
					}
				}
			}
		}

		const arrOrder = [];

		for(let strMac in arrNICDetails)
		{
			if(objNICMatches[strMac] !== undefined)
			{
				arrOrder.push(objNICMatches[strMac]);
			}
		}

		if(arrOrder.length !== 0)
		{
			strBootmgrCleanupCommand += `efibootmgr -o ${arrOrder.join(",")} \n`;
		}

		return await CommandExecutor.execCommandProcess(strBootmgrCleanupCommand);
	}


	/**
	 * @param {Object} objServer
	 * @returns {string}
	 */
	static async networkingIntelCleanup(objServer)
	{
		let strNetworkingCleanupCommand = `
			intelToUpdate=\`/bin/bootutil64e | tail -n +10 | grep -v PXE | wc -l\`
			if [ $intelToUpdate -eq "0" ]; then
				echo "No Intel NICs needed update"
			else
				/bin/bootutil64e -quiet  -up=pxe -all -file=/opt/bdk/BootIMG.FLB
			fi
		`;

		const objNICDetails = objServer.NICDetails;
		const arrEmulexOC10GbMACs = [];
		let bFoundEmulexOC10Gb = false;

		for(let strMACAddr in objNICDetails)
		{
			const objNIC = objNICDetails[strMACAddr];

			if(
				objNIC["vendor"] === "Emulex Corporation"
				&& objNIC["product"] === "OneConnect 10Gb NIC (be3)"
			)
			{
				const strLinedMAC = strMACAddr.replace(/:/g, "-");

				/**
				 * enable NIC personality
				 * disable SRIOV due to performance reasons (only 100Mbps could be achived on those links)
				 * make sure pxe is enabled on that device
				 */
				strNetworkingCleanupCommand += `
					/usr/sbin/hbacmd ChangePersonality ${strLinedMAC} NIC 1
					/usr/sbin/hbacmd SRIOVEnable ${strLinedMAC} 0
					/usr/sbin/hbacmd EnableBootCode ${strLinedMAC} E PXE
				`;

				bFoundEmulexOC10Gb = true;
				arrEmulexOC10GbMACs.push(strLinedMAC);
			}

			// proprietary Bigstep NICs from Emulex
			if(
				objNIC["vendor"] === "Emulex Corporation"
				&& objNIC["product"] === "OneConnect NIC (Skyhawk)"
			)
			{
				// Commented, because it takes a very long time - approx 8 mins for all 4 NICs.
				// const strLinedMAC = strMACAddr.replace(/:/g, "-");
				// strNetworkingCleanupCommand += `/usr/sbin/hbacmd ChangePersonality ${strLinedMAC} iSCSI\n`;
			}
		}

		if(bFoundEmulexOC10Gb)
		{
			strNetworkingCleanupCommand += `# /bin/bash /bin/emulex_nc522_upgrade_firmware.sh "${arrEmulexOC10GbMACs.join(" ")}"\n`;
		}

		let strCommandOutput = "";

		try
		{
			strCommandOutput = await CommandExecutor.execCommandProcess(strNetworkingCleanupCommand);
		}
		catch(error)
		{
			// if(!strCommandOutput.includes("not supported for this device"))
			// {
			// 	throw error;
			// }
		}

		return strCommandOutput;
	}


	/**
	 * @param {AgentIdentity} agentIdentity
	 * @param {Object} objServer
	 * @param {boolean} bCleanOnlyBootloader = false
	 * @returns {string}
	 */
	static async diskCleanup(agentIdentity, objServer, bCleanOnlyBootloader = false)
	{
		let strDiskCleanupCommand = "";
		const arrDisks = objServer.diskCollection;

		if(
			agentIdentity
			&&
			(
				agentIdentity.environment === AgentIdentity.DEVELOPMENT
				|| agentIdentity.environment === AgentIdentity.INTEGRATION
			)
		)
		{
			if (agentIdentity.environment === AgentIdentity.DEVELOPMENT)
			{
				return "";
			}

			let nBlockSize = 67108864;

			if(arrDisks && arrDisks.length !== 0)
			{
				for(let objDisk of arrDisks)
				{
					// Must not be wiped for DELL
					if(
						objDisk.server_disk_model === "IDSDM"
						&& objDisk.server_disk_size_gb < 20
					)
					{
						continue;
					}
	
					if(objDisk.server_disk_type === BDKServerCleanup.DISK_TYPE_NVME)
					{
						strDiskCleanupCommand += `
							dd if=/dev/urandom of=${objDisk.device_path} bs=${nBlockSize} count=1 conv=notrunc &
						`;
					}
					else
					{
						strDiskCleanupCommand += `
							dd if=/dev/urandom of=/dev/${objDisk.server_disk_label} bs=${nBlockSize} count=1 conv=notrunc &
						`;
					}
				}
	
				strDiskCleanupCommand += `
					while [ $(ps -eo pid,cmd | grep "dd if=/dev/urandom" | grep -v grep | wc -l) -ne "0" ]; do
						echo Waiting for dd to finish...
						sleep 1m;
					done
					echo Finished all disks
				`;
			}

			return await CommandExecutor.execCommandProcess(strDiskCleanupCommand);
		}

		if(arrDisks && arrDisks.length !== 0)
		{
			// For benchmark look at the bottom of this file
			let nBlockSize = 67108864;
			let strDDCount = bCleanOnlyBootloader ? "count=1" : "";

			for(let objDisk of arrDisks)
			{
				// Must not be wiped for DELL
				if(
					objDisk.server_disk_model === "IDSDM"
					&& objDisk.server_disk_size_gb < 20
				)
				{
					continue;
				}

				// strDiskCleanupCommand += `
				// 	dd if=/dev/urandom of=${objDisk.device_path} bs=10M conv=notrunc &
				// `;

				if(objDisk.server_disk_type === BDKServerCleanup.DISK_TYPE_NVME)
				{
					strDiskCleanupCommand += `
						nvme format --ses=2 ${objDisk.device_path}
						if [ $? == "0" ]; then
							echo Drive wiping succesfully performed with nvme tool, no need for dd wiping
						else
							nvme format --ses=1 ${objDisk.device_path}
							if [ $? == "0" ]; then
								echo Drive wiping succesfully performed with nvme tool, no need for dd wiping
							else
								dd if=/dev/urandom of=${objDisk.device_path} bs=${nBlockSize} ${strDDCount} conv=notrunc &
							fi
						fi
					`;
				}
				else
				{
					if(objDisk.server_disk_type === BDKServerCleanup.DISK_TYPE_HDD)
					{
						if(!bCleanOnlyBootloader)
						{
							nBlockSize = 4096;
						}
					}

					strDiskCleanupCommand += `
						sg_sanitize --quick --crypto --wait /dev/${objDisk.server_disk_label}
						if [ $? == "0" ]; then
							echo Drive wiping succesfully performed with sg_sanitize tool, no need for dd wiping
						else
							dd if=/dev/urandom of=/dev/${objDisk.server_disk_label} bs=${nBlockSize} ${strDDCount} conv=notrunc &
						fi
					`;
				}
			}

			strDiskCleanupCommand += `
				while [ $(ps -eo pid,cmd | grep "dd if=/dev/urandom" | grep -v grep | wc -l) -ne "0" ]; do
					echo Waiting for dd to finish...
					sleep 1m;
				done
				echo Finished all disks
			`;
		}

		return await CommandExecutor.execCommandProcess(strDiskCleanupCommand);
	}


	/**
	 * Runs a batch code containing auxiliary functions which are called during cleaning.
	 * (iLO related cleanup)
	 * 
	 * @returns {string}
	 */
	static async _runServerCleanupAuxiliaryFunctionsBatch()
	{
		const strCommand = `
			ILOREST=/sbin/ilorest
			function ilo_login() {
				logincount=1
				while true; do
					timeout --kill-after=13 10 \${ILOREST} login >/dev/null 2>&1
					if [ $? -eq 0 ]; then
						echo 'Login successfull'
						break
					fi
					echo \"Waiting for ILO to be available, try #\${logincount}...\"
					sleep 5
					logincount=\$((logincount+1))
					if [ \${logincount} -eq 40 ]; then
						echo \"Unable to login on iLO. Exiting\"
						exit
					fi
				done
			}
		`;

		const strCommandOutput = await CommandExecutor.execCommandProcess(strCommand);
		return strCommandOutput;
	}

	static get DISK_TYPE_NVME()
	{
		return "NVME";
	}
	
	static get DISK_TYPE_HDD()
	{
		return "HDD";
	}

	/**
	 * Server interface add-on device role HBA.
	 */
	static get SERVER_INTERFACE_ADD_ON_HBA()
	{
		return "hba";
	}

	/**
	 * HP vendor label from database
	 */
	static get HP_VENDOR()
	{
		return "HP";
	}

	/**
	 * HPE vendor label from database
	 */
	static get HPE_VENDOR()
	{
		return "HPE";
	}
}

module.exports = BDKServerCleanup;


// HDD
// block size : transfer rate
//        512 : 224 MB/s
//       1024 : 338 MB/s
//       2048 : 391 MB/s
//       4096 : 504 MB/s
//       8192 : 436 MB/s
//      16384 : 392 MB/s
//      32768 : 386 MB/s
//      65536 : 376 MB/s
//     131072 : 383 MB/s
//     262144 : 417 MB/s
//     524288 : 378 MB/s
//    1048576 : 393 MB/s
//    2097152 : 372 MB/s
//    4194304 : 445 MB/s
//    8388608 : 371 MB/s
//   16777216 : 349 MB/s
//   33554432 : 358 MB/s
//   67108864 : 386 MB/s
